# -*- coding: utf-8 -*-

from datetime import datetime

from Acquisition import aq_inner

from zope.interface import alsoProvides
from zope.component import getMultiAdapter

from kss.core import kssaction
from plone.memoize.instance import memoize 
from plone.app.layout.viewlets import common
from plone.app.kss.plonekssview import PloneKSSView
from plone.app.layout.globals.interfaces import IViewView
from collective.captcha.browser.captcha import Captcha

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFPlone import Batch

from garbas.forum.browser import BrowserContentView
from garbas.forum.interfaces import IForum
from garbas.forum.interfaces import IForumTopic
from garbas.forum.interfaces import IForumNotification
from garbas.forum import ForumMessageFactory as _


TOPIC_SUCCESS     = _(u'topic_success', default='Topic was successfuly added.')
TOPIC_TITLE_ERROR = _(u'topic_title_error', default='Subject is required. Please correct.')
TOPIC_TEXT_ERROR  = _(u'topic_text_error', default='Text is required. Please correct.')
CAPTCHA_ERROR    = _(u'captcha_error', default=u'Captcha is required. Please correct.')
CAPTCHA_ERROR2   = _(u'captcha_error2', default=u'Captcha is incorrect.')


class ForumView(BrowserContentView):
    """ forum main view """

    render = ViewPageTemplateFile('templates/forum.pt')

    def __init__(self, context, request, view=None):
        BrowserContentView.__init__(self, context, request, view)
        self.request.set('disable_border', 1)

    def render_template(self):
        return 'garbas.forum.forum_view' 
  
    def is_manager(self):
        membership = getToolByName(self.context, 'portal_membership')
        member = membership.getAuthenticatedMember()
        return 'Manager' in member.getRolesInContext(self.context)

    def has_permissions_to_addtopic(self):
        membership = getToolByName(self.context, 'portal_membership')
        return membership.checkPermission("Forum: Add ForumTopic", self.context) and \
                  self.context.allow_addtopic

    def has_topics(self):
      return len(self.topics()) > 0

    def has_subforums(self):
      return len(self.subforums()) > 0

    @memoize
    def topics(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        
        sorting = getattr(self.request, 'garbas.forum.topics.sorting', 'forum_last_post_date')
        sorting_order = getattr(self.request, 'garbas.forum.topics.sorting_order', 'reverse') 

        b_size = getattr(self.request, 'b_size', 20)
        b_start = getattr(self.request, 'b_start', 0)

        results = [
                  dict(
                      title = topic.Title,
                      url = self.context.absolute_url()+'/'+str(topic.id),
                      author = topic.Creator,
                      author_url = self.site_url+'/author/'+str(topic.Creator),
                      created = topic.created,
                      replies = topic.forum_posts,
                      last_post_author = topic.forum_last_post_author,
                      last_post_author_url = self.site_url+'/author/'+str(topic.forum_last_post_author), 
                      last_post_url = self.site_url+str(topic.forum_last_post_url),
                      last_post_date = topic.forum_last_post_date,
                  )

                  for topic in catalog(
                      object_provides=IForumTopic.__identifier__,
                      path=dict(query='/'.join(self.context.getPhysicalPath()),
                                depth=1),
                      sort_on=sorting,
                      sort_order=sorting_order,)
               ]
        return Batch(results, b_size, b_start, orphan=1)


    @memoize
    def subforums(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        uncategorizes_forums = _(u'uncategorizes_forums', default='Uncategorized')

        sorting = getattr(self.request, 'garbas.forum.subforums-sorting', 'forum_last_post_date')
        sorting_order = getattr(self.request, 'garbas.forum.subforums-sorting_order', 'reverse') 

        forums = {}
        for forum in catalog(object_provides=IForum.__identifier__,
                             path=dict(query='/'.join(self.context.getPhysicalPath()),depth=1),
                             sort_on=sorting,
                             sort_order=sorting_order,):
            if forum.forum_category in self.context.categories: 
                category = forum.forum_category
            else: 
                category = uncategorizes_forums
            
            if not forums.has_key(category):
                forums[category] = []
            forums[category].append(dict(
                    title = forum.Title,
                    url = self.context.absolute_url()+'/'+str(forum.id),
                    description = forum.Description,
                    topics = forum.forum_topics,
                    posts = forum.forum_posts+forum.forum_topics,
                    last_post_author = forum.forum_last_post_author,
                    last_post_author_url = self.site_url+'/author/'+str(forum.forum_last_post_author), 
                    last_post_url = self.site_url+str(forum.forum_last_post_url),
                    last_post_date = forum.forum_last_post_date,
                ))
        categories = self.context.categories + (uncategorizes_forums, )
        subforums = []
        for category in categories:
            if not forums.has_key(category): continue
            subforums.append(dict(
                title=category,
                forums=forums[category],))
        
        return subforums
            

class ForumAddTopic(BrowserContentView):
    """ add topic form """
    
    errors = dict()
    render = ViewPageTemplateFile('templates/add_topic.pt')
   
    def __init__(self, context, request, view=None):
        BrowserContentView.__init__(self, context, request, view)
        self.request.set('disable_border', 1)

    def render_template(self):
        return 'garbas.forum.add_topic' 

    def update(self):
        """ all action happens here """

        if getattr(self.request, 'form.button.submit', None):
            # VALIDATE FIELDS
            self.errors = {}
            for method in dir(self):
                if method[:9] == 'validate_':
                    method = getattr(self, method, False)
                    if method:
                        errors = method()
                        if errors:
                            self.errors.update(errors)

            if not self.errors:
                title = getattr(self.request, 'topic_title', None)
                text = getattr(self.request, 'topic_text', None)
                transforms = getToolByName(self.context, 'portal_transforms')
                text = transforms('text_to_html', text)

                now = datetime.today()
                topic_id = self.context.invokeFactory('ForumTopic', 
                        id=now.strftime('forumtopic_%Y-%m-%d.')+str(now.microsecond),
                        title=title, 
                        text=text)
                topic = getattr(self.context, topic_id)
                topic._renameAfterCreation()
                
                membership = getToolByName(self.context, 'portal_membership')
                member = str(membership.getAuthenticatedMember())
                IForumNotification(topic).subscribe(member)

                # FIXME :: portal_status_message si not working
                self.request.response.redirect(
                    topic.absolute_url() + '/' + \
                    '?portal_status_messages=' + TOPIC_SUCCESS)
    
    def validate_title(self):
        if 'topic_title' not in self.request or \
           not self.request['topic_title']:
            self.errors['topic_title'] = TOPIC_TITLE_ERROR

    def validate_text(self):
        if 'topic_text' not in self.request or \
           not self.request['topic_text']:
            self.errors['topic_text'] = TOPIC_TEXT_ERROR

    def validate_captcha(self):
        if not self.is_anonymous(): return
        if 'captcha' not in self.request or \
           not self.request['captcha']:
            self.errors['captcha'] = CAPTCHA_ERROR 
        elif not Captcha(self.context, self.request).verify(self.request['captcha']):
            self.errors['captcha'] = CAPTCHA_ERROR2
        
    def is_anonymous(self):
        membership = getToolByName(self.context, 'portal_membership')
        return membership.isAnonymousUser()

    def captcha(self):
        return Captcha(self.context, self.request)






# -*- coding: utf-8 -*-

from zope.interface import implements

from Products.Archetypes import atapi

from Products.ATContentTypes.content.document import ATDocumentSchema
from Products.ATContentTypes.content.document import ATDocument
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from garbas.forum.config import PROJECTNAME

from garbas.forum import ForumMessageFactory as _
from garbas.forum.interfaces import IForumPost



PostSchema = ATDocumentSchema.copy()

PostSchema['title'].storage = atapi.AnnotationStorage()
PostSchema['title'].widget.visible = {'edit':'invisible', 'view':'invisible'}
PostSchema['description'].storage = atapi.AnnotationStorage()
PostSchema['description'].widget.visible = {'edit':'invisible', 'view':'invisible'}
PostSchema['text'].storage = atapi.AnnotationStorage()

finalizeATCTSchema(PostSchema)


class ForumPost(ATDocument):
    """forum post content
    """
    implements(IForumPost)
    
    portal_type = "ForumPost"
    _at_rename_after_creation = True
    schema = PostSchema
    
    title = atapi.ATFieldProperty('title')
    text = atapi.ATFieldProperty('text')
   
atapi.registerType(ForumPost, PROJECTNAME)







from Acquisition import aq_inner

from BTrees.OOBTree import OOBTree
from persistent.list import PersistentList
from zope.interface import implements
from zope.annotation.interfaces import IAnnotations

from Products.CMFCore.utils import getToolByName

from garbas.forum.interfaces import IForumNotification


NOTIFICATION_KEY = 'garbas.forum.notification'

class ForumNotification(object):
    """ notification """

    implements(IForumNotification)

    def __init__(self, context):
        self.context = aq_inner(context)
        self.annotations = IAnnotations(self.context)
        notifications = self.annotations.get(NOTIFICATION_KEY, None)
        if notifications is None:
            self.annotations[NOTIFICATION_KEY] = PersistentList()

    def notification_emails(self, ignore=[]):
        notification_list = []
        membership = getToolByName(self.context, 'portal_membership')
        for member in self.annotations[NOTIFICATION_KEY]:
            if member not in ignore:
                member = membership.getMemberById(member)
                if member:
                    member_email = member.getProperty('email')
                    if member_email:
                        notification_list.append(member_email)
        return notification_list

    def subscribe(self, member):
        if member not in self.annotations[NOTIFICATION_KEY]:
            if member:
                self.annotations[NOTIFICATION_KEY].append(member)
    
    def unsubscribe(self, member):
        if member in self.annotations[NOTIFICATION_KEY]:
            self.annotations[NOTIFICATION_KEY].remove(member)






# -*- coding: utf-8 -*-

from datetime import datetime

from zope.i18n import translate
from zope.component import getUtility
from zope.component import getMultiAdapter
from plone.memoize.instance import memoize 
from collective.captcha.browser.captcha import Captcha

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import Batch
from Products.CMFCore.interfaces import ISiteRoot

from email.MIMEMultipart import MIMEMultipart
from email.MIMEBase import MIMEBase
from email.MIMEText import MIMEText
from email.Utils import COMMASPACE, formatdate
from email import Encoders

from garbas.forum.browser import BrowserContentView
from garbas.forum.interfaces import IForumPost
from garbas.forum.interfaces import IForumNotification
from garbas.forum import ForumMessageFactory as _


POST_SUCCESS     = _(u'post_success', default='Post was successfuly added.')
POST_TITLE_ERROR = _(u'post_title_error', default='Subject is required. Please correct.')
POST_TEXT_ERROR  = _(u'post_text_error', default='Text is required. Please correct.')
CAPTCHA_ERROR    = _(u'captcha_error', default=u'Captcha is required. Please correct.')
CAPTCHA_ERROR2   = _(u'captcha_error2', default=u'Captcha is incorrect.')



class ForumTopicView(BrowserContentView):
    """ topic view """

    errors = dict()
    render = ViewPageTemplateFile('templates/topic.pt')
   
    def __init__(self, context, request, view=None):
        BrowserContentView.__init__(self, context, request, view)
        self.request.set('disable_border', 1)

    def render_template(self):
        return 'garbas.forum.topic_view' 

    def is_manager(self):
        membership = getToolByName(self.context, 'portal_membership')
        member = membership.getAuthenticatedMember()
        return 'Manager' in member.getRolesInContext(self.context)
    
    def has_permissions_to_addposts(self):
        membership = getToolByName(self.context, 'portal_membership')
        return membership.checkPermission("Forum: Add ForumPost", self.context)

    def has_posts(self):
        return len(self.posts()) > 0

    @memoize
    def posts(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        membership = getToolByName(self.context, 'portal_membership')

        b_size = getattr(self.request, 'b_size', 20)
        b_start = getattr(self.request, 'b_start', 0)

        result = [ dict( title        = 'RE: ' + self.context.title,
                         UID          = item.UID,
                         author       = item.Creator and item.Creator or 'Anon',
                         author_url   = self.site_url+'/author/'+str(item.Creator),
                         portrait_url = membership.getPersonalPortrait(item.Creator).absolute_url(),
                         created      = item.created, 
                         text         = item.getObject().text )
                   for item in catalog(
                         object_provides = IForumPost.__identifier__,
                         path            = dict(query='/'.join(self.context.getPhysicalPath()),
                                                depth=1),
                         sort_on         = 'modified',)]

        return Batch([dict( title        = self.context.title,
                            UID          = self.context.UID(),
                            author       = self.context.Creator(),
                            author_url   = self.site_url+'/author/'+str(self.context.Creator()),
                            portrait_url = membership.getPersonalPortrait(self.context.Creator()).absolute_url(),
                            created      = self.context.CreationDate(),
                            text         = self.context.text )] + result, b_size, b_start, orphan=1)


class ForumAddPost(BrowserContentView):
    """ add post form """
    
    errors = dict()
    render = ViewPageTemplateFile('templates/add_post.pt')
   
    def __init__(self, context, request, view=None):
        BrowserContentView.__init__(self, context, request, view)
        self.request.set('disable_border', 1)

    def render_template(self):
        return 'garbas.forum.add_post' 

    def update(self):
        """ all action happens here """

        if getattr(self.request, 'unsubscribe', None):
            member = '' 
            self.unsubscribe(member)


        elif getattr(self.request, 'form.button.submit', None):

            # VALIDATE FIELDS
            self.errors = {}
            for method in dir(self):
                if method[:9] == 'validate_':
                    method = getattr(self, method, False)
                    if method:
                        errors = method()
                        if errors: 
                            self.errors.update(errors)

            if not self.errors:
                transforms = getToolByName(self.context, 'portal_transforms')
                text = getattr(self.request, 'post_text', None)
                text = transforms('text_to_html', text)

                now = datetime.today()
                post_id = self.context.invokeFactory('ForumPost', 
                          id=now.strftime('forumpost_%Y-%m-%d.')+str(now.microsecond),
                          text=text)
                post = getattr(self.context, post_id)
                post._renameAfterCreation()

                portal = getUtility(ISiteRoot)
                language = getToolByName(self.context, 'portal_languages')
                membership = getToolByName(self.context, 'portal_membership')
                localization_code = language.getLanguageBindings()[0]
                mail_encoding = portal.getProperty('email_charset')
                portal_from = str(portal.getProperty('email_from_address')) + \
                                  '<' + str(portal.getProperty('email_from_name ')) + '>'

                member = str(membership.getAuthenticatedMember())
                if member:
                    IForumNotification(self.context).subscribe(member)
  
                mail = MIMEMultipart()
                mail.set_charset(mail_encoding)
                mail['From'] = portal_from
                mail['Date'] = formatdate(localtime=True)
                mail['Subject'] = translate('notify', 
                    domain = 'garbas.forum',
                    target_language = localization_code,
                    context = self.context).encode(mail_encoding)
                mail.attach( MIMEText( translate( 'mail_add_new_member_text',
                    domain = 'garbas.forum',
                    mapping = dict(),
                    target_language = localization_code,
                    context = self.context).encode(mail_encoding)))

                host = self.context.MailHost
                emails = IForumNotification(self.context).notification_emails()
                for email in emails:
                    mail['To'] = email
                    host.send(mail.as_string().decode(mail_encoding).encode(mail_encoding))
            
                # FIXME :: portal_status_message si not working
                self.request.response.redirect(
                    self.site_url + '/' + \
                    post.absolute_url() + '/' + \
                    '?portal_status_messages=' + POST_SUCCESS)

    def validate_text(self):
        if 'post_text' not in self.request or \
           not self.request['post_text']:
            self.errors['post_text'] = POST_TEXT_ERROR

    def validate_captcha(self):
        membership = getToolByName(self.context, 'portal_membership')
        if not membership.isAnonymousUser(): return
        if 'captcha' not in self.request or \
           not self.request['captcha']:
            self.errors['captcha'] = CAPTCHA_ERROR 
        elif not Captcha(self.context, self.request).verify(self.request['captcha']):
            self.errors['captcha'] = CAPTCHA_ERROR2
    
    def is_anonymous(self):
        membership = getToolByName(self.context, 'portal_membership')
        return membership.isAnonymousUser()

    def captcha(self):
        return Captcha(self.context, self.request)

    def unsubscribe(self, member):
        IForumNotification(self.context).unsubscribe(member)







# -*- coding: utf-8 -*-

from zope.interface import implements

from Products.Archetypes import atapi
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from garbas.forum.content.base import ForumFolder
from garbas.forum.content.base import ForumFolderSchema
from garbas.forum.interfaces import IForumTopic
from garbas.forum.config import PROJECTNAME
from garbas.forum import ForumMessageFactory as _


TopicSchema = ForumFolderSchema.copy() + atapi.Schema((
    atapi.TextField(
        name='text',
        default_output_type = 'text/x-html-safe',
        widget = atapi.RichWidget(
            label = _(u'topic_label_text', default=u'Text'),
            description = _(u'topic_help_text', default='Write text.'),
            rows = 25,
            allow_file_upload = zconf.ATDocument.allow_document_upload
        ),
        storage=atapi.AnnotationStorage(),
        validators = ('isTidyHtmlWithCleanup',),
        searchable=True,
    ),
))
TopicSchema['description'].widget.visible = {'edit':'invisible', 'view':'invisible'}
finalizeATCTSchema(TopicSchema)


class ForumTopic(ForumFolder):
    """forum topic content"""

    implements(IForumTopic)

    portal_type = "ForumTopic"
    schema = TopicSchema
    
    title       = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')
    text        = atapi.ATFieldProperty('text')
   
atapi.registerType(ForumTopic, PROJECTNAME)





# -*- coding: utf-8 -*-

from zope.interface import implements
from zope.component import getMultiAdapter
from AccessControl import ClassSecurityInfo

from Products.Archetypes import atapi

from Products.CMFCore.utils import getToolByName
from Products.ATContentTypes.content.folder import ATBTreeFolderSchema
from Products.ATContentTypes.content.folder import ATBTreeFolder
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from garbas.forum import ForumMessageFactory as _
from garbas.forum.interfaces import IForumTopic
from garbas.forum.interfaces import IForumPost
from garbas.forum.interfaces import IForum


ForumFolderSchema = ATBTreeFolderSchema.copy()
ForumFolderSchema['title'].storage = atapi.AnnotationStorage()
ForumFolderSchema['description'].storage = atapi.AnnotationStorage()


class ForumFolder(ATBTreeFolder):
    """forum folder"""

    _at_rename_after_creation = True

    schema = ForumFolderSchema 
    security = ClassSecurityInfo()

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')

    def last_post(self):
        catalog = getToolByName(self, 'portal_catalog')
        result = catalog(object_provides=IForumPost.__identifier__,
                              path=dict(query='/'.join(self.getPhysicalPath())),
                              sort_on='created',
                              sort_order='reverse',
                              sort_limit=1,)[:1]
        if result: 
          return result[0]

    def last_topic(self):
        catalog = getToolByName(self, 'portal_catalog')
        result = catalog(object_provides=IForumTopic.__identifier__,
                              path=dict(query='/'.join(self.getPhysicalPath())),
                              sort_on='created',
                              sort_order='reverse',
                              sort_limit=1,)[:1]
        if result: 
          return result[0]

    security.declarePublic('forum_category')
    def forum_category(self):
        """forum category"""
        return getattr(self, 'category', None)
            
    security.declarePublic('forum_topics')
    def forum_topics(self):
        """number of all topics"""
        catalog = getToolByName(self, 'portal_catalog')
        return len(catalog(object_provides=IForumTopic.__identifier__,
                                path=dict(query='/'.join(self.getPhysicalPath()),),))
            
    security.declarePublic('forum_posts')
    def forum_posts(self):
        """number of all posts"""
        catalog = getToolByName(self, 'portal_catalog')
        return len(catalog(object_provides=IForumPost.__identifier__,
                                path=dict(query='/'.join(self.getPhysicalPath()),),))
            
    security.declarePublic('forum_last_post_date')
    def forum_last_post_date(self):
        """last post date"""
        last_post_date = getattr(self.last_post(), 'created', None)
        if not last_post_date:
            last_post_date = getattr(self.last_topic(), 'created', None)
        return last_post_date


    security.declarePublic('forum_last_post_author')
    def forum_last_post_author(self):
        """last post author"""
        last_post_author = getattr(self.last_post(), 'Creator', None)
        if not last_post_author:
            last_post_author = getattr(self.last_topic(), 'Creator', None)
        return last_post_author

    security.declarePublic('forum_last_post_url')
    def forum_last_post_url(self):
        """last post url"""
        last_post = self.last_post()
        if not last_post:
            last_post = self.last_topic()
        if last_post: 
            portal = getToolByName(self, 'portal_url').getPortalObject()
            site_url = portal.absolute_url()
            return last_post.getObject().absolute_url()[len(site_url):]
        return None

def reindex_for_topics(context, event):
    """reindex all sub-forum folders"""
    parent = context.getParentNode()
    while parent.__providedBy__(IForum):
        parent.reindexObject(['forum_topics',])
        parent = parent.getParentNode()

def reindex_for_posts(context, event):
    """reindex all sub-forum folders"""
    parent = context.getParentNode()
    while parent.__providedBy__(IForum) or parent.__providedBy__(IForumTopic):
        parent.reindexObject(['forum_posts','forum_last_post_author', 'forum_last_post_date', 'forum_last_post_url',])
        parent = parent.getParentNode()

    
